@@ -100,10 +100,6 @@ def reload_constraints():
 reload_constraints()
 
 
-# Maximum slo segments in buffer
-MAX_BUFFERED_SLO_SEGMENTS = 10000
-
-
 # By default the maximum number of allowed headers depends on the number of max
 # allowed metadata settings plus a default value of 36 for swift internally
 # generated headers and regular http headers.  If for some reason this is not
@@ -408,9 +404,10 @@ def check_name_format(req, name, target_type):
         raise HTTPPreconditionFailed(
             request=req,
             body='%s name cannot be empty' % target_type)
-    if isinstance(name, six.text_type):
-        name = name.encode('utf-8')
-    if b'/' in name:
+    if six.PY2:
+        if isinstance(name, six.text_type):
+            name = name.encode('utf-8')
+    if '/' in name:
         raise HTTPPreconditionFailed(
             request=req,
             body='%s name cannot contain slashes' % target_type)
