@@ -32,7 +32,6 @@ from eventlet.green import socket, ssl, os as green_os
 import six
 from six import BytesIO
 from six import StringIO
-from six.moves import configparser
 
 from swift.common import utils, constraints
 from swift.common.storage_policy import BindPortsCache
@@ -41,7 +40,7 @@ from swift.common.swob import Request, wsgi_quote, wsgi_unquote, \
 from swift.common.utils import capture_stdio, disable_fallocate, \
     drop_privileges, get_logger, NullLogger, config_true_value, \
     validate_configuration, get_hub, config_auto_int_value, \
-    reiterate
+    reiterate, NicerInterpolation
 
 SIGNUM_TO_NAME = {getattr(signal, n): n for n in dir(signal)
                   if n.startswith('SIG') and '_' not in n}
@@ -56,23 +55,6 @@ except (ImportError, NotImplementedError):
     CPU_COUNT = 1
 
 
-if not six.PY2:
-    # In general, we haven't really thought much about interpolation in
-    # configs. Python's default ConfigParser has always supported it, though,
-    # so *we* got it "for free". Unfortunatley, since we "supported"
-    # interpolation, we have to assume there are deployments in the wild that
-    # use it, and try not to break them. So, do what we can to mimic the py2
-    # behavior of passing through values like "1%" (which we want to support
-    # for fallocate_reserve).
-    class NicerInterpolation(configparser.BasicInterpolation):
-        def before_get(self, parser, section, option, value, defaults):
-            if '%(' not in value:
-                return value
-            return super(NicerInterpolation, self).before_get(
-                parser, section, option, value, defaults)
-    configparser.ConfigParser._DEFAULT_INTERPOLATION = NicerInterpolation()
-
-
 class NamedConfigLoader(loadwsgi.ConfigLoader):
     """
     Patch paste.deploy's ConfigLoader so each context object will know what
@@ -80,6 +62,8 @@ class NamedConfigLoader(loadwsgi.ConfigLoader):
     """
 
     def get_context(self, object_type, name=None, global_conf=None):
+        if not six.PY2:
+            self.parser._interpolation = NicerInterpolation()
         context = super(NamedConfigLoader, self).get_context(
             object_type, name=name, global_conf=global_conf)
         context.name = name
@@ -453,17 +437,21 @@ class SwiftHttpProtocol(wsgi.HttpProtocol):
             return ''
 
     def parse_request(self):
+        # Need to track the bytes-on-the-wire for S3 signatures -- eventlet
+        # would do it for us, but since we rewrite the path on py3, we need to
+        # fix it ourselves later.
+        self.__raw_path_info = None
+
         if not six.PY2:
             # request lines *should* be ascii per the RFC, but historically
             # we've allowed (and even have func tests that use) arbitrary
             # bytes. This breaks on py3 (see https://bugs.python.org/issue33973
             # ) but the work-around is simple: munge the request line to be
-            # properly quoted. py2 will do the right thing without this, but it
-            # doesn't hurt to re-write the request line like this and it
-            # simplifies testing.
+            # properly quoted.
             if self.raw_requestline.count(b' ') >= 2:
                 parts = self.raw_requestline.split(b' ', 2)
                 path, q, query = parts[1].partition(b'?')
+                self.__raw_path_info = path
                 # unquote first, so we don't over-quote something
                 # that was *correctly* quoted
                 path = wsgi_to_bytes(wsgi_quote(wsgi_unquote(
@@ -485,7 +473,11 @@ class SwiftHttpProtocol(wsgi.HttpProtocol):
     if not six.PY2:
         def get_environ(self, *args, **kwargs):
             environ = wsgi.HttpProtocol.get_environ(self, *args, **kwargs)
+            environ['RAW_PATH_INFO'] = bytes_to_wsgi(
+                self.__raw_path_info)
             header_payload = self.headers.get_payload()
+            if isinstance(header_payload, list) and len(header_payload) == 1:
+                header_payload = header_payload[0].get_payload()
             if header_payload:
                 # This shouldn't be here. We must've bumped up against
                 # https://bugs.python.org/issue37093
