@@ -228,6 +228,21 @@ class TestSymlinkMiddleware(TestSymlinkMiddlewareBase):
         self.assertIn(('Content-Location', '/v1/a/c1/o'), headers)
         self.assertIn(b'does not exist', body)
 
+    def test_symlink_simple_put_error(self):
+        self.app.register('HEAD', '/v1/a/c1/o',
+                          swob.HTTPInternalServerError, {}, 'bad news')
+        req = Request.blank('/v1/a/c/symlink', method='PUT',
+                            headers={
+                                'X-Symlink-Target': 'c1/o',
+                                'X-Symlink-Target-Etag': 'not-tgt-etag',
+                            }, body='')
+        status, headers, body = self.call_sym(req)
+        self.assertEqual(status, '500 Internal Error')
+        # this is a PUT response; so if we have a content-length...
+        self.assertGreater(int(dict(headers)['Content-Length']), 0)
+        # ... we better have a body!
+        self.assertIn(b'Internal Error', body)
+
     def test_symlink_put_with_prevalidated_etag(self):
         self.app.register('PUT', '/v1/a/c/symlink', swob.HTTPCreated, {})
         req = Request.blank('/v1/a/c/symlink', method='PUT', headers={
